@@ -8,7 +8,19 @@ import logging
 import binascii
 import contextlib
 import collections
-import cStringIO as StringIO
+
+# six
+import six
+
+# TODO replace with six.byte2int, when it is released
+if six.PY3:
+    from io import BytesIO as BytesIO
+    def byte2int(b): return b
+    def stringlike(x): return isinstance(x, (str, bytes))
+else:
+    from cStringIO import StringIO as BytesIO
+    def byte2int(b): return ord(b)
+    def stringlike(x): return isinstance(x, basestring)
 
 # PyCrypto
 import Crypto.Util
@@ -28,41 +40,46 @@ class IntegrityError(ValueError):
 SER_COMPACT = 0
 SER_BINARY  = 1
 
-COMPACT_DIGITS = ('!#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
-                          '[]^_abcdefghijklmnopqrstuvwxyz{|}~')
+COMPACT_DIGITS = (b'!#$%&()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ' +
+                          b'[]^_abcdefghijklmnopqrstuvwxyz{|}~')
 R_COMPACT_DIGITS = {}  # TODO is a tuple/list faster?
 for i, c in enumerate(COMPACT_DIGITS):
     R_COMPACT_DIGITS[c] = i
 
 def serialize_number(x, fmt=SER_BINARY, outlen=None):
     """ Serializes `x' to a string of length `outlen' in format `fmt' """
-    ret = ''
+    ret = b''
     if fmt == SER_BINARY:
         while x:
             x, r = divmod(x, 256)
-            ret = chr(r) + ret
+            ret = six.int2byte(int(r)) + ret
         if outlen is not None:
             assert len(ret) <= outlen
-            ret = ret.rjust(outlen, '\0')
+            ret = ret.rjust(outlen, b'\0')
         return ret
     assert fmt == SER_COMPACT
     while x:
         x, r = divmod(x, len(COMPACT_DIGITS))
-        ret = COMPACT_DIGITS[r] + ret
+        ret = COMPACT_DIGITS[r:r+1] + ret
     if outlen is not None:
         assert len(ret) <= outlen
-        ret = ret.rjust(outlen, COMPACT_DIGITS[0])
+        ret = ret.rjust(outlen, COMPACT_DIGITS[0:1])
     return ret
 
 def deserialize_number(s, fmt=SER_BINARY):
     """ Deserializes a number from a string `s' in format `fmt' """
     ret = gmpy.mpz(0)
     if fmt == SER_BINARY:
+        if isinstance(s, six.text_type):
+            raise ValueError("Encode `s` to a bytestring yourself to"+
+                         " prevent problems with different default encodings")
         for c in s:
             ret *= 256
-            ret += ord(c)
+            ret += byte2int(c)
         return ret
     assert fmt == SER_COMPACT
+    if isinstance(s, six.text_type):
+        s = s.encode('ascii')
     for c in s:
         ret *= len(COMPACT_DIGITS)
         ret += R_COMPACT_DIGITS[c]
@@ -70,11 +87,11 @@ def deserialize_number(s, fmt=SER_BINARY):
 
 def get_serialized_number_len(x, fmt=SER_BINARY):
     if fmt == SER_BINARY:
-        return (x.numdigits(2) + 7) / 8
+        return (x.numdigits(2) + 7) // 8
     assert fmt == SER_COMPACT
     res = 0
     while x != 0:
-        x = x / len(COMPACT_DIGITS)
+        x = x // len(COMPACT_DIGITS)
         res += 1
     return res
 
@@ -85,7 +102,7 @@ def mod_issquare(a, p):
     """ Returns whether `a' is a square modulo p """
     if not a:
         return True
-    p1 = p / 2
+    p1 = p // 2
     p2 = pow(a, p1, p)
     return p2 == 1
 
@@ -130,79 +147,79 @@ raw_curve_parameters = collections.namedtuple('raw_curve_parameters',
             ('name', 'a', 'b', 'm', 'base_x', 'base_y', 'order', 'cofactor'))
 RAW_CURVES = {
     18 : ("secp112r1",
-        "db7c2abf62e35e668076bead2088", 
-        "659ef8ba043916eede8911702b22", 
-        "db7c2abf62e35e668076bead208b",
-        "09487239995a5ee76b55f9c2f098",
-        "a89ce5af8724c0a23e0e0ff77500", 
-        "db7c2abf62e35e7628dfac6561c5", 1),
+        b"db7c2abf62e35e668076bead2088",
+        b"659ef8ba043916eede8911702b22",
+        b"db7c2abf62e35e668076bead208b",
+        b"09487239995a5ee76b55f9c2f098",
+        b"a89ce5af8724c0a23e0e0ff77500",
+        b"db7c2abf62e35e7628dfac6561c5", 1),
     20: ("secp128r1",
-        "fffffffdfffffffffffffffffffffffc", 
-        "e87579c11079f43dd824993c2cee5ed3", 
-        "fffffffdffffffffffffffffffffffff",
-        "161ff7528b899b2d0c28607ca52c5b86", 
-        "cf5ac8395bafeb13c02da292dded7a83",
-        "fffffffe0000000075a30d1b9038a115", 1),
-    25: ("secp160r1", 
-        "ffffffffffffffffffffffffffffffff7ffffffc",
-        "1c97befc54bd7a8b65acf89f81d4d4adc565fa45",
-        "ffffffffffffffffffffffffffffffff7fffffff",
-        "4a96b5688ef573284664698968c38bb913cbfc82",
-        "23a628553168947d59dcc912042351377ac5fb32",
-        "0100000000000000000001f4c8f927aed3ca752257", 1),
+        b"fffffffdfffffffffffffffffffffffc",
+        b"e87579c11079f43dd824993c2cee5ed3",
+        b"fffffffdffffffffffffffffffffffff",
+        b"161ff7528b899b2d0c28607ca52c5b86",
+        b"cf5ac8395bafeb13c02da292dded7a83",
+        b"fffffffe0000000075a30d1b9038a115", 1),
+    25: ("secp160r1",
+        b"ffffffffffffffffffffffffffffffff7ffffffc",
+        b"1c97befc54bd7a8b65acf89f81d4d4adc565fa45",
+        b"ffffffffffffffffffffffffffffffff7fffffff",
+        b"4a96b5688ef573284664698968c38bb913cbfc82",
+        b"23a628553168947d59dcc912042351377ac5fb32",
+        b"0100000000000000000001f4c8f927aed3ca752257", 1),
     30: ("secp192r1/nistp192",
-        "fffffffffffffffffffffffffffffffefffffffffffffffc",
-        "64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1", 
-        "fffffffffffffffffffffffffffffffeffffffffffffffff",
-        "188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012",
-        "07192b95ffc8da78631011ed6b24cdd573f977a11e794811",
-        "ffffffffffffffffffffffff99def836146bc9b1b4d22831", 1),
+        b"fffffffffffffffffffffffffffffffefffffffffffffffc",
+        b"64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1",
+        b"fffffffffffffffffffffffffffffffeffffffffffffffff",
+        b"188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012",
+        b"07192b95ffc8da78631011ed6b24cdd573f977a11e794811",
+        b"ffffffffffffffffffffffff99def836146bc9b1b4d22831", 1),
     35: ("secp224r1/nistp224",
-        "fffffffffffffffffffffffffffffffefffffffffffffffffffffffe",
-        "b4050a850c04b3abf54132565044b0b7d7bfd8ba270b39432355ffb4",
-        "ffffffffffffffffffffffffffffffff000000000000000000000001",
-        "b70e0cbd6bb4bf7f321390b94a03c1d356c21122343280d6115c1d21",
-        "bd376388b5f723fb4c22dfe6cd4375a05a07476444d5819985007e34",
-        "ffffffffffffffffffffffffffff16a2e0b8f03e13dd29455c5c2a3d", 1),
+        b"fffffffffffffffffffffffffffffffefffffffffffffffffffffffe",
+        b"b4050a850c04b3abf54132565044b0b7d7bfd8ba270b39432355ffb4",
+        b"ffffffffffffffffffffffffffffffff000000000000000000000001",
+        b"b70e0cbd6bb4bf7f321390b94a03c1d356c21122343280d6115c1d21",
+        b"bd376388b5f723fb4c22dfe6cd4375a05a07476444d5819985007e34",
+        b"ffffffffffffffffffffffffffff16a2e0b8f03e13dd29455c5c2a3d", 1),
     40: ("secp256r1/nistp256",
-        "ffffffff00000001000000000000000000000000fffffffffffffffffffffffc", 
-        "5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b", 
-        "ffffffff00000001000000000000000000000000ffffffffffffffffffffffff",
-        "6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296", 
-        "4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5", 
-        "ffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551", 1),
+        b"ffffffff00000001000000000000000000000000fffffffffffffffffffffffc",
+        b"5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b",
+        b"ffffffff00000001000000000000000000000000ffffffffffffffffffffffff",
+        b"6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296",
+        b"4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5",
+        b"ffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551", 1),
     60: ("secp384r1/nistp384",
-        "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffe"+
-            "ffffffff0000000000000000fffffffc",
-        "b3312fa7e23ee7e4988e056be3f82d19181d9c6efe8141120314088f5013875a"+
-            "c656398d8a2ed19d2a85c8edd3ec2aef", 
-        "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffe"+
-            "ffffffff0000000000000000ffffffff",
-        "aa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a38"+
-            "5502f25dbf55296c3a545e3872760ab7",
-        "3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c0"+
-            "0a60b1ce1d7e819d7a431d7c90ea0e5f", 
-        "ffffffffffffffffffffffffffffffffffffffffffffffffc7634d81f4372ddf"+
-            "581a0db248b0a77aecec196accc52973", 1),
+        b"fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffe"+
+            b"ffffffff0000000000000000fffffffc",
+        b"b3312fa7e23ee7e4988e056be3f82d19181d9c6efe8141120314088f5013875a"+
+            b"c656398d8a2ed19d2a85c8edd3ec2aef",
+        b"fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffe"+
+            b"ffffffff0000000000000000ffffffff",
+        b"aa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a38"+
+            b"5502f25dbf55296c3a545e3872760ab7",
+        b"3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c0"+
+            b"0a60b1ce1d7e819d7a431d7c90ea0e5f",
+        b"ffffffffffffffffffffffffffffffffffffffffffffffffc7634d81f4372ddf"+
+            b"581a0db248b0a77aecec196accc52973", 1),
     81: ("secp521r1/nistp521",
-        "01ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
-            "ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
-            "fffffffc",
-        "0051953eb9618e1c9a1f929a21a0b68540eea2da725b99b315f3b8b489918ef1"+
-            "09e156193951ec7e937b1652c0bd3bb1bf073573df883d2c34f1ef451fd4"+
-            "6b503f00",
-        "01ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
-            "ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
-            "ffffffff",
-        "00c6858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d"+
-            "3dbaa14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31"+
-            "c2e5bd66",
-        "011839296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e"+
-            "662c97ee72995ef42640c550b9013fad0761353c7086a272c24088be9476"+
-            "9fd16650",
-        "01ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
-            "fffa51868783bf2f966b7fcc0148f709a5d03bb5c9b8899c47aebb6fb71e"+
-            "91386409", 1)
+        b"01ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
+            b"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
+            b"fffffffc",
+        b"0051953eb9618e1c9a1f929a21a0b68540eea2da725b99b315f3b8b489918ef1"+
+            b"09e156193951ec7e937b1652c0bd3bb1bf073573df883d2c34f1ef451fd4"+
+            b"6b503f00",
+        b"01ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
+            b"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
+            b"ffffffff",
+        b"00c6858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d"+
+            b"3dbaa14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31"+
+            b"c2e5bd66",
+        b"011839296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e"+
+            b"662c97ee72995ef42640c550b9013fad0761353c7086a272c24088be9476"+
+            b"9fd16650",
+        b"01ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff"+
+            b"fffa51868783bf2f966b7fcc0148f709a5d03bb5c9b8899c47aebb6fb71e"+
+            b"91386409", 1)
     }
 
 # Arithmetic on elliptic curves
@@ -284,7 +301,7 @@ class JacobianPoint(object):
         y = (y * t2) % m
         y = (y - t1) % m
         return JacobianPoint(x=x, y=y, z=z, curve=self.curve)
-        
+
     def __repr__(self):
         return "<JacobianPoint (%s, %s, %s) of %s>" % (
                             self.x, self.y, self.z, self.curve.name)
@@ -349,17 +366,27 @@ class AffinePoint(object):
 
     def __nonzero__(self):
         return bool(self.x or self.y)
+    __bool__ = __nonzero__
+
     def __repr__(self):
         return "<AffinePoint (%s, %s) of %s>" % (
                             self.x, self.y, self.curve.name)
+    def __eq__(self, other):
+        if not isinstance(other, AffinePoint):
+            return False
+        return self.x == other.x and self.y == other.y
+    def __ne__(self, other):
+        return not (self == other)
     def __str__(self):
         return self.to_string(SER_COMPACT)
-    def to_string(self, fmt=SER_BINARY):
+    def to_bytes(self, fmt=SER_BINARY):
         outlen = (self.curve.pk_len_compact if fmt == SER_COMPACT
                         else self.curve.pk_len_bin)
         if self._point_compress():
             return serialize_number(self.x + self.curve.m, fmt, outlen)
         return serialize_number(self.x, fmt, outlen)
+    def to_string(self, fmt=SER_BINARY):
+        return self.to_bytes(fmt).decode()
     def _point_compress(self):
         return self.y.getbit(0) == 1
     def _ECIES_KDF(self, R):
@@ -436,17 +463,22 @@ class PubKey(object):
 
     def encrypt(self, s, mac_bytes=10):
         """ Encrypt `s' for this pubkey. """
-        out = StringIO.StringIO()
+        if isinstance(s, six.text_type):
+            raise ValueError("Encode `s` to a bytestring yourself to"+
+                         " prevent problems with different default encodings")
+        out = BytesIO()
         with self.encrypt_to(out, mac_bytes) as f:
             f.write(s)
         return out.getvalue()
 
+    def to_bytes(self, fmt=SER_BINARY):
+        return self.p.to_bytes(fmt)
     def to_string(self, fmt=SER_BINARY):
         return self.p.to_string(fmt)
     def __str__(self):
-        return str(self.p)
+        return self.to_string(SER_COMPACT)
     def __repr__(self):
-        return "<PubKey %s>" % str(self)
+        return "<PubKey %s>" % unicode(self)
 
 class PrivKey(object):
     """ A secret exponent """
@@ -460,7 +492,9 @@ class PrivKey(object):
         yield ctx
         ctx.read()
     def decrypt(self, s, mac_bytes=10):
-        instream = StringIO.StringIO(s)
+        if isinstance(s, six.text_type):
+            raise ValueError("s should be bytes")
+        instream = BytesIO(s)
         with self.decrypt_from(instream, mac_bytes) as f:
             return f.read()
     def sign(self, h, sig_format=SER_BINARY):
@@ -485,7 +519,7 @@ class PrivKey(object):
         s = 0
         while s == 0:
             while r == 0:
-                buf = cprng.encrypt('\0'*self.curve.order_len_bin)
+                buf = cprng.encrypt(b'\0'*self.curve.order_len_bin)
                 k = self.curve._buf_to_exponent(buf)
                 p1 = self.curve.base * k
                 r = p1.x % order
@@ -508,7 +542,7 @@ class EncryptionContext(object):
         self.mac_bytes = mac_bytes
         key, R = p._ECIES_encryption()
         self.h = hmac.new(key[32:], digestmod=hashlib.sha256)
-        f.write(R.to_string(SER_BINARY))
+        f.write(R.to_bytes(SER_BINARY))
         ctr = Crypto.Util.Counter.new(128, initial_value=0)
         self.cipher = Crypto.Cipher.AES.new(key[:32],
                 Crypto.Cipher.AES.MODE_CTR, counter=ctr)
@@ -557,9 +591,20 @@ class DecryptionContext(object):
 class Curve(object):
     """ Represents a Elliptic Curve """
 
+    @staticmethod
+    def by_name_substring(substring):
+        substring = substring.lower()
+        candidates = []
+        for raw_curve in six.itervalues(RAW_CURVES):
+            if substring in raw_curve[0]:
+                candidates.append(raw_curve)
+        if len(candidates) != 1:
+            raise KeyError
+        return Curve(candidates[0])
+
     @staticmethod
     def by_name(name):
-        for raw_curve in RAW_CURVES.itervalues():
+        for raw_curve in six.itervalues(RAW_CURVES):
             if raw_curve[0] == name:
                 return Curve(raw_curve)
         raise KeyError
@@ -572,7 +617,7 @@ class Curve(object):
 
             Use `Curve.by_pk_len' instead """
         r = raw_curve_parameters(*raw_curve_params)
-        
+
         # Store domain parameters
         self.name = r.name
         self.a = deserialize_number(binascii.unhexlify(r.a), SER_BINARY)
@@ -593,7 +638,7 @@ class Curve(object):
                                 (self.order * self.order) - 1, SER_BINARY)
         self.sig_len_compact = get_serialized_number_len(
                                 (self.order * self.order) - 1, SER_COMPACT)
-        self.dh_len_bin = min((self.order.numdigits(2) / 2 + 7) / 8, 32)
+        self.dh_len_bin = min((self.order.numdigits(2) // 2 + 7) // 8, 32)
         self.dh_len_compact = get_serialized_number_len(
                                 2 ** self.dh_len_bin - 1, SER_COMPACT)
         self.elem_len_bin = get_serialized_number_len(self.m, SER_BINARY)
@@ -632,7 +677,7 @@ class Curve(object):
         ctr = Crypto.Util.Counter.new(128, initial_value=0)
         cipher = Crypto.Cipher.AES.new(h,
                     Crypto.Cipher.AES.MODE_CTR, counter=ctr)
-        buf = cipher.encrypt('\0' * self.order_len_bin)
+        buf = cipher.encrypt(b'\0' * self.order_len_bin)
         return self._buf_to_exponent(buf)
     def _buf_to_exponent(self, buf):
         a = deserialize_number(buf, SER_BINARY)
@@ -641,6 +686,9 @@ class Curve(object):
     def passphrase_to_pubkey(self, passphrase):
         return PubKey(self.base * self.passphrase_to_privkey(passphrase).e)
     def passphrase_to_privkey(self, passphrase):
+        if isinstance(passphrase, six.text_type):
+            raise ValueError("Encode `passphrase` to a bytestring yourself to"+
+                         " prevent problems with different default encodings")
         h = _passphrase_to_hash(passphrase)
         return PrivKey(self.hash_to_exponent(h), self)
     @contextlib.contextmanager
@@ -649,7 +697,7 @@ class Curve(object):
         yield ctx
         ctx.read()
     def decrypt(self, s, privkey, mac_bytes=10):
-        instream = StringIO.StringIO(s)
+        instream = BytesIO(s)
         with self.decrypt_from(instream, privkey, mac_bytes) as f:
             return f.read()
 
@@ -677,10 +725,10 @@ def encrypt_file(in_path_or_file, out_path_or_file, pk, pk_format=SER_COMPACT,
     close_in, close_out = False, False
     in_file, out_file = in_path_or_file, out_path_or_file
     try:
-        if isinstance(in_path_or_file, basestring):
+        if stringlike(in_path_or_file):
             in_file = open(in_path_or_file, 'rb')
             close_in = True
-        if isinstance(out_path_or_file, basestring):
+        if stringlike(out_path_or_file):
             out_file = open(out_path_or_file, 'wb')
             close_out = True
         _encrypt_file(in_file, out_file, pk, pk_format, mac_bytes, chunk_size)
@@ -694,10 +742,10 @@ def decrypt_file(in_path_or_file, out_path_or_file, passphrase,
     close_in, close_out = False, False
     in_file, out_file = in_path_or_file, out_path_or_file
     try:
-        if isinstance(in_path_or_file, basestring):
+        if stringlike(in_path_or_file):
             in_file = open(in_path_or_file, 'rb')
             close_in = True
-        if isinstance(out_path_or_file, basestring):
+        if stringlike(out_path_or_file):
             out_file = open(out_path_or_file, 'wb')
             close_out = True
         _decrypt_file(in_file, out_file, passphrase, curve, mac_bytes,
@@ -731,12 +779,18 @@ def _decrypt_file(in_file, out_file, passphrase, curve='secp160r1',
 def verify(s, sig, pk, sig_format=SER_COMPACT, pk_format=SER_COMPACT):
     """ Verifies that `sig' is a signature of pubkey `pk' for the
         message `s'. """
+    if isinstance(s, six.text_type):
+        raise ValueError("Encode `s` to a bytestring yourself to"+
+                     " prevent problems with different default encodings")
     curve = Curve.by_pk_len(len(pk))
     p = curve.pubkey_from_string(pk, pk_format)
     return p.verify(hashlib.sha512(s).digest(), sig, sig_format)
 
 def sign(s, passphrase, sig_format=SER_COMPACT, curve='secp160r1'):
     """ Signs `s' with passphrase `passphrase' """
+    if isinstance(s, six.text_type):
+        raise ValueError("Encode `s` to a bytestring yourself to"+
+                     " prevent problems with different default encodings")
     curve = Curve.by_name(curve)
     privkey = curve.passphrase_to_privkey(passphrase)
     return privkey.sign(hashlib.sha512(s).digest(), sig_format)
