@@ -2,33 +2,36 @@ import unittest
 import tempfile
 
 import seccure
+import six
 
 class TestMain(unittest.TestCase):
     def test_passphrase_to_pubkey(self):
-        self.assertEqual(str(seccure.passphrase_to_pubkey('test')),
+        self.assertEqual(str(seccure.passphrase_to_pubkey(b'test')),
                                 '*jMVCU^[QC&q*v_8C1ZAFBAgD')
-        self.assertEqual(str(seccure.passphrase_to_pubkey('my private key')),
+        self.assertEqual(str(seccure.passphrase_to_pubkey(b'my private key')),
                                 '8W;>i^H0qi|J&$coR5MFpR*Vn')
+        self.assertRaises(ValueError, seccure.passphrase_to_pubkey,
+                            six.u('test'))
     def test_encrypt(self):
-        msg = 'My private message'
-        pw = 'my private key'
+        msg = b'My private message'
+        pw = b'my private key'
         self.assertEqual(seccure.decrypt(seccure.encrypt(msg,
                         str(seccure.passphrase_to_pubkey(pw))),
-                            'my private key'), msg)
+                            b'my private key'), msg)
     def test_verify(self):
-        msg = 'This message will be signed\n'
-        sig = '$HPI?t(I*1vAYsl$|%21WXND=6Br*[>k(OR9B!GOwHqL0s+3Uq'
+        msg = b'This message will be signed\n'
+        sig = b'$HPI?t(I*1vAYsl$|%21WXND=6Br*[>k(OR9B!GOwHqL0s+3Uq'
         pubkey = '8W;>i^H0qi|J&$coR5MFpR*Vn'
         self.assertTrue(seccure.verify(msg, sig, pubkey))
     def test_sign(self):
-        msg = 'This message will be signed\n'
-        pw = 'my private key'
+        msg = b'This message will be signed\n'
+        pw = b'my private key'
         self.assertEqual(seccure.sign(msg, pw),
-                '$HPI?t(I*1vAYsl$|%21WXND=6Br*[>k(OR9B!GOwHqL0s+3Uq')
+                b'$HPI?t(I*1vAYsl$|%21WXND=6Br*[>k(OR9B!GOwHqL0s+3Uq')
 
     def test_encrypt_file_named(self):
-        msg = 'My private message'
-        pw = 'my private key'
+        msg = b'My private message'
+        pw = b'my private key'
         pubkey = '8W;>i^H0qi|J&$coR5MFpR*Vn'
 
         in_file = tempfile.NamedTemporaryFile()
@@ -46,8 +49,8 @@ class TestMain(unittest.TestCase):
         self.assertEqual(msg, decrypted_file.read())
 
     def test_encrypt_file(self):
-        msg = 'My private message'
-        pw = 'my private key'
+        msg = b'My private message'
+        pw = b'my private key'
         pubkey = '8W;>i^H0qi|J&$coR5MFpR*Vn'
 
         encrypted_file = tempfile.NamedTemporaryFile()
